@@ -15,7 +15,7 @@ if [ "$TRAVIS_EVENT_TYPE" = "pull_request" ]; then
 fi
 
 test/lint/git-subtree-check.sh src/crypto/ctaes
-test/lint/git-subtree-check.sh src/secp256k1
+test/lint/git-subtree-check.sh src/secp256k1-zkp
 test/lint/git-subtree-check.sh src/univalue
 test/lint/git-subtree-check.sh src/leveldb
 test/lint/git-subtree-check.sh src/crc32c
