@@ -1,18 +1,17 @@
-// Copyright (c) 2016-2018 The Bitcoin Core developers
+// Copyright (c) 2016-2019 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
-#include <iostream>
 
 #include <bench/bench.h>
 #include <bloom.h>
 
-static void RollingBloom(benchmark::State& state)
+static void RollingBloom(benchmark::Bench& bench)
 {
     CRollingBloomFilter filter(120000, 0.000001);
     std::vector<unsigned char> data(32);
     uint32_t count = 0;
-    while (state.KeepRunning()) {
+    bench.run([&] {
         count++;
         data[0] = count;
         data[1] = count >> 8;
@@ -25,7 +24,16 @@ static void RollingBloom(benchmark::State& state)
         data[2] = count >> 8;
         data[3] = count;
         filter.contains(data);
-    }
+    });
 }
 
-BENCHMARK(RollingBloom, 1500 * 1000);
+static void RollingBloomReset(benchmark::Bench& bench)
+{
+    CRollingBloomFilter filter(120000, 0.000001);
+    bench.run([&] {
+        filter.reset();
+    });
+}
+
+BENCHMARK(RollingBloom);
+BENCHMARK(RollingBloomReset);
