@@ -45,5 +45,5 @@ void MemMMR::Rewind(const uint64_t numLeaves)
 {
     assert(numLeaves <= m_leaves.size());
     m_leaves.resize(numLeaves);
-    m_hashes.resize(GetNextLeafIdx().GetPosition());
+    m_hashes.resize(GetNumNodes());
 }
\ No newline at end of file
