@@ -27,7 +27,7 @@
     </message>
     <message>
         <source>Delete the currently selected address from the list</source>
-        <translation>Kustuta märgistatud aadress loetelust</translation>
+        <translation>Kustuta valitud aadress nimekirjast</translation>
     </message>
     <message>
         <source>Enter address or label to search</source>
@@ -66,12 +66,8 @@
         <translation>Vastuvõtvad aadressid</translation>
     </message>
     <message>
-        <source>These are your Litecoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
-        <translation>Need on sinu Litecoin aadressid maksete saatmiseks. Ennem müntide saatmist kontrolli alati summat ja makse saaja aadressi.</translation>
-    </message>
-    <message>
-        <source>These are your Litecoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
-        <translation>Need on sinu Litecoin aadressid sisenevate maksete vastu võtmiseks. Soovitav on iga tehingu tarbeks kasutada uut aadressi.</translation>
+        <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
+        <translation>Need on sinu Bitcoin aadressid maksete saatmiseks. Ennem müntide saatmist kontrolli alati summat ja makse saaja aadressi.</translation>
     </message>
     <message>
         <source>&amp;Copy Address</source>
@@ -106,7 +102,7 @@
     <name>AddressTableModel</name>
     <message>
         <source>Label</source>
-        <translation>Märge</translation>
+        <translation>Silt</translation>
     </message>
     <message>
         <source>Address</source>
@@ -114,7 +110,7 @@
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märge puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
 </context>
 <context>
@@ -125,19 +121,19 @@
     </message>
     <message>
         <source>Enter passphrase</source>
-        <translation>Sisesta salafraas</translation>
+        <translation>Sisesta parool</translation>
     </message>
     <message>
         <source>New passphrase</source>
-        <translation>Uus salafraas</translation>
+        <translation>Uus parool</translation>
     </message>
     <message>
         <source>Repeat new passphrase</source>
-        <translation>Korda salafraasi</translation>
+        <translation>Korda uut parooli</translation>
     </message>
     <message>
-        <source>Enter the new passphrase to the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
-        <translation>Sisesta uus salafraas rahakotti.&lt;br/&gt;Kasuta salafraasi millles on&lt;b&gt;kümme või rohkem juhuslikku sümbolit&lt;b&gt;,või&lt;b&gt;kaheksa või rohkem sõna&lt;b/&gt;.</translation>
+        <source>Show passphrase</source>
+        <translation>Näita salafraasi</translation>
     </message>
     <message>
         <source>Encrypt wallet</source>
@@ -161,19 +157,15 @@
     </message>
     <message>
         <source>Change passphrase</source>
-        <translation>Vaheta salafraasi</translation>
-    </message>
-    <message>
-        <source>Enter the old passphrase and new passphrase to the wallet.</source>
-        <translation>Sisesta vana salafraas ja uus salafraas rahakotti.</translation>
+        <translation>Muuda parooli</translation>
     </message>
     <message>
         <source>Confirm wallet encryption</source>
         <translation>Kinnita rahakoti krüpteerimine.</translation>
     </message>
     <message>
-        <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR LITECOINS&lt;/b&gt;!</source>
-        <translation>Hoiatus:Kui sa krüpteerid oma rahakoti ja kaotad salafraasi, siis sa&lt;b&gt;KAOTAD OMA LITECOINID&lt;/b&gt;!</translation>
+        <source>Warning: If you encrypt your wallet and lose your passphrase, you will &lt;b&gt;LOSE ALL OF YOUR BITCOINS&lt;/b&gt;!</source>
+        <translation>Hoiatus:Kui sa krüpteerid oma rahakoti ja kaotad salafraasi, siis sa&lt;b&gt;KAOTAD OMA BITCOINID&lt;/b&gt;!</translation>
     </message>
     <message>
         <source>Are you sure you wish to encrypt your wallet?</source>
@@ -183,17 +175,45 @@
         <source>Wallet encrypted</source>
         <translation>Rahakott krüpteeritud</translation>
     </message>
+    <message>
+        <source>Enter the new passphrase for the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
+        <translation>Sisesta rahakotile uus salafraas.&lt;br/&gt;Kasuta salafraasi millles on&lt;b&gt;kümme või rohkem juhuslikku sümbolit&lt;b&gt;,või&lt;b&gt;kaheksa või rohkem sõna&lt;b/&gt;.</translation>
+    </message>
+    <message>
+        <source>Enter the old passphrase and new passphrase for the wallet.</source>
+        <translation>Sisesta rahakoti vana salafraas ja uus salafraas.</translation>
+    </message>
+    <message>
+        <source>Remember that encrypting your wallet cannot fully protect your bitcoins from being stolen by malware infecting your computer.</source>
+        <translation>Pea meeles, et rahakoti krüpteerimine ei välista bitcoinide vargust, kui sinu arvuti on nakatunud pahavaraga.</translation>
+    </message>
+    <message>
+        <source>Wallet to be encrypted</source>
+        <translation>Krüpteeritav rahakott</translation>
+    </message>
+    <message>
+        <source>Your wallet is about to be encrypted. </source>
+        <translation>Rahakott krüpteeritakse.</translation>
+    </message>
+    <message>
+        <source>Your wallet is now encrypted. </source>
+        <translation>Rahakott krüpteeritud.</translation>
+    </message>
+    <message>
+        <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
+        <translation>TÄHTIS: Kõik varasemad rahakoti varundfailid tuleks üle kirjutada äsja loodud krüpteeritud rahakoti failiga. Turvakaalutlustel tühistatakse krüpteerimata rahakoti failid alates uue, krüpteeritud rahakoti, kasutusele võtust.</translation>
+    </message>
     <message>
         <source>Wallet encryption failed</source>
         <translation>Rahakoti krüpteerimine ebaõnnestus</translation>
     </message>
     <message>
         <source>Wallet encryption failed due to an internal error. Your wallet was not encrypted.</source>
-        <translation>Rahakoti krüpteerimine ebaõnnestus sisemise tõrke tõttu. Sinu rahakott ei ole krüpteeritud.</translation>
+        <translation>Rahakoti krüpteerimine ebaõnnestus sisemise vea tõttu. Sinu rahakotti ei krüpteeritud.</translation>
     </message>
     <message>
         <source>The supplied passphrases do not match.</source>
-        <translation>Sisestatud salafraasid ei kattu.</translation>
+        <translation>Sisestatud paroolid ei kattu.</translation>
     </message>
     <message>
         <source>Wallet unlock failed</source>
@@ -209,7 +229,7 @@
     </message>
     <message>
         <source>Wallet passphrase was successfully changed.</source>
-        <translation>Rahakoti salafraas on edukalt vahetatud.</translation>
+        <translation>Rahakoti parooli vahetus õnnestus.</translation>
     </message>
     <message>
         <source>Warning: The Caps Lock key is on!</source>
@@ -222,7 +242,11 @@
         <source>IP/Netmask</source>
         <translation>IP/Võrgumask</translation>
     </message>
-    </context>
+    <message>
+        <source>Banned Until</source>
+        <translation>Blokeeritud kuni</translation>
+    </message>
+</context>
 <context>
     <name>BitcoinGUI</name>
     <message>
@@ -231,7 +255,7 @@
     </message>
     <message>
         <source>Synchronizing with network...</source>
-        <translation>Võrgusünkimine...</translation>
+        <translation>Võrguga sünkroniseerimine...</translation>
     </message>
     <message>
         <source>&amp;Overview</source>
@@ -255,12 +279,16 @@
     </message>
     <message>
         <source>Quit application</source>
-        <translation>Väljumine</translation>
+        <translation>Välju rakendusest</translation>
     </message>
     <message>
         <source>&amp;About %1</source>
         <translation>&amp;Teave %1</translation>
     </message>
+    <message>
+        <source>Show information about %1</source>
+        <translation>Näita informatsiooni %1 kohta</translation>
+    </message>
     <message>
         <source>About &amp;Qt</source>
         <translation>Teave &amp;Qt kohta</translation>
@@ -273,6 +301,10 @@
         <source>&amp;Options...</source>
         <translation>&amp;Valikud...</translation>
     </message>
+    <message>
+        <source>Modify configuration options for %1</source>
+        <translation>Muuda %1 seadeid</translation>
+    </message>
     <message>
         <source>&amp;Encrypt Wallet...</source>
         <translation>&amp;Krüpteeri Rahakott</translation>
@@ -290,12 +322,24 @@
         <translation>Ava &amp;URI...</translation>
     </message>
     <message>
-        <source>Reindexing blocks on disk...</source>
-        <translation>Kettal olevate blokkide re-indekseerimine...</translation>
+        <source>Create Wallet...</source>
+        <translation>Loo rahakott</translation>
     </message>
     <message>
-        <source>Send coins to a Litecoin address</source>
-        <translation>Saada münte Litecoini aadressile</translation>
+        <source>Create a new wallet</source>
+        <translation>Loo uus rahakott</translation>
+    </message>
+    <message>
+        <source>Wallet:</source>
+        <translation>Rahakott:</translation>
+    </message>
+    <message>
+        <source>Reindexing blocks on disk...</source>
+        <translation>Kõvakettal olevate plokkide reindekseerimine...</translation>
+    </message>
+    <message>
+        <source>Send coins to a Bitcoin address</source>
+        <translation>Saada münte Bitcoini aadressile</translation>
     </message>
     <message>
         <source>Backup wallet to another location</source>
@@ -305,22 +349,10 @@
         <source>Change the passphrase used for wallet encryption</source>
         <translation>Rahakoti krüpteerimise salafraasi muutmine</translation>
     </message>
-    <message>
-        <source>&amp;Debug window</source>
-        <translation>&amp;Silumise aken</translation>
-    </message>
-    <message>
-        <source>Open debugging and diagnostic console</source>
-        <translation>Ava debugimise ja diagnostika konsool</translation>
-    </message>
     <message>
         <source>&amp;Verify message...</source>
         <translation>&amp;Kontrolli sõnumit...</translation>
     </message>
-    <message>
-        <source>Litecoin</source>
-        <translation>Litecoin</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Saada</translation>
@@ -342,12 +374,12 @@
         <translation>Krüpteeri oma rahakoti privaatvõtmed</translation>
     </message>
     <message>
-        <source>Sign messages with your Litecoin addresses to prove you own them</source>
-        <translation>Omandi tõestamiseks allkirjasta sõnumid oma Litecoini aadressiga</translation>
+        <source>Sign messages with your Bitcoin addresses to prove you own them</source>
+        <translation>Omandi tõestamiseks allkirjasta sõnumid oma Bitcoini aadressiga</translation>
     </message>
     <message>
-        <source>Verify messages to ensure they were signed with specified Litecoin addresses</source>
-        <translation>Kinnita sõnumid kindlustamaks et need allkirjastati määratud Litecoini aadressiga</translation>
+        <source>Verify messages to ensure they were signed with specified Bitcoin addresses</source>
+        <translation>Kinnita sõnumid kindlustamaks et need allkirjastati määratud Bitcoini aadressiga</translation>
     </message>
     <message>
         <source>&amp;File</source>
@@ -366,20 +398,16 @@
         <translation>Vahelehe tööriistariba</translation>
     </message>
     <message>
-        <source>Request payments (generates QR codes and litecoin: URIs)</source>
-        <translation>Loo maksepäring (genereerib QR koodid ja litecoini: URId)</translation>
-    </message>
-    <message>
-        <source>Open a litecoin: URI or payment request</source>
-        <translation>Ava litecoini: URI või maksepäring</translation>
+        <source>Request payments (generates QR codes and bitcoin: URIs)</source>
+        <translation>Loo maksepäring (genereerib QR koodid ja bitcoini: URId)</translation>
     </message>
     <message>
         <source>&amp;Command-line options</source>
-        <translation>Käsurea valikud</translation>
+        <translation>&amp;Käsurea valikud</translation>
     </message>
     <message numerus="yes">
-        <source>%n active connection(s) to Litecoin network</source>
-        <translation><numerusform>%n aktiivne ühendus Litecoini võrku</numerusform><numerusform>%n aktiivset ühendust Litecoini võrku</numerusform></translation>
+        <source>%n active connection(s) to Bitcoin network</source>
+        <translation><numerusform>%n aktiivne ühendus Bitcoini võrku</numerusform><numerusform>%n aktiivset ühendust Bitcoini võrku</numerusform></translation>
     </message>
     <message>
         <source>Indexing blocks on disk...</source>
@@ -395,7 +423,7 @@
     </message>
     <message>
         <source>%1 behind</source>
-        <translation>%1 maas</translation>
+        <translation>%1 ajast maas</translation>
     </message>
     <message>
         <source>Last received block was generated %1 ago.</source>
@@ -403,11 +431,11 @@
     </message>
     <message>
         <source>Transactions after this will not yet be visible.</source>
-        <translation>Peale seda ei ole tehingud veel nähtavad.</translation>
+        <translation>Hilisemad transaktsioonid ei ole veel nähtavad.</translation>
     </message>
     <message>
         <source>Error</source>
-        <translation>Tõrge</translation>
+        <translation>Viga</translation>
     </message>
     <message>
         <source>Warning</source>
@@ -492,7 +520,7 @@
     </message>
     <message>
         <source>Amount:</source>
-        <translation>Summa:</translation>
+        <translation>Kogus</translation>
     </message>
     <message>
         <source>Fee:</source>
@@ -552,11 +580,11 @@
     </message>
     <message>
         <source>Copy amount</source>
-        <translation>Kopeeri summa</translation>
+        <translation>Kopeeri kogus</translation>
     </message>
     <message>
         <source>Copy transaction ID</source>
-        <translation>Kopeeri tehingu ID</translation>
+        <translation>Kopeeri transaktsiooni ID</translation>
     </message>
     <message>
         <source>Copy quantity</source>
@@ -592,13 +620,19 @@
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märgis puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
     <message>
         <source>(change)</source>
         <translation>(vahetusraha)</translation>
     </message>
 </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -626,8 +660,8 @@
         <translation>Muuda saatvat aadressi</translation>
     </message>
     <message>
-        <source>The entered address "%1" is not a valid Litecoin address.</source>
-        <translation>Sisestatud aadress "%1" ei ole korrektne Litecoin aadress.</translation>
+        <source>The entered address "%1" is not a valid Bitcoin address.</source>
+        <translation>Sisestatud aadress "%1" ei ole korrektne Bitcoin aadress.</translation>
     </message>
     <message>
         <source>Could not unlock wallet.</source>
@@ -660,15 +694,15 @@
     <name>Intro</name>
     <message>
         <source>Welcome</source>
-        <translation>Teretulemast</translation>
+        <translation>Tere tulemast</translation>
     </message>
     <message>
-        <source>Litecoin</source>
-        <translation>Litecoin</translation>
+        <source>Bitcoin</source>
+        <translation>Bitcoin</translation>
     </message>
     <message>
         <source>Error</source>
-        <translation>Tõrge</translation>
+        <translation>Viga</translation>
     </message>
     </context>
 <context>
@@ -688,23 +722,14 @@
     </context>
 <context>
     <name>OpenURIDialog</name>
-    <message>
-        <source>Open URI</source>
-        <translation>Ava URI</translation>
-    </message>
     <message>
         <source>URI:</source>
         <translation>URI:</translation>
     </message>
-    <message>
-        <source>Select payment request file</source>
-        <translation>Vali maksepäringu fail</translation>
-    </message>
-    <message>
-        <source>Select payment request file to open</source>
-        <translation>Vali maksepäringu fail mida avada</translation>
-    </message>
 </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -736,8 +761,8 @@
         <translation>Ekspert</translation>
     </message>
     <message>
-        <source>Automatically open the Litecoin client port on the router. This only works when your router supports UPnP and it is enabled.</source>
-        <translation>Litecoini kliendi pordi automaatne avamine ruuteris. Toimib, kui sinu ruuter aktsepteerib UPnP ühendust.</translation>
+        <source>Automatically open the Bitcoin client port on the router. This only works when your router supports UPnP and it is enabled.</source>
+        <translation>Bitcoini kliendi pordi automaatne avamine ruuteris. Toimib, kui sinu ruuter aktsepteerib UPnP ühendust.</translation>
     </message>
     <message>
         <source>Map port using &amp;UPnP</source>
@@ -821,7 +846,7 @@
     </message>
     <message>
         <source>Error</source>
-        <translation>Tõrge</translation>
+        <translation>Viga</translation>
     </message>
     <message>
         <source>The supplied proxy address is invalid.</source>
@@ -835,8 +860,8 @@
         <translation>Vorm</translation>
     </message>
     <message>
-        <source>The displayed information may be out of date. Your wallet automatically synchronizes with the Litecoin network after a connection is established, but this process has not completed yet.</source>
-        <translation>Kuvatav info ei pruugi olla ajakohane. Ühenduse loomisel süngitakse sinu rahakott automaatselt Liteconi võrgustikuga, kuid see toiming on hetkel lõpetamata.</translation>
+        <source>The displayed information may be out of date. Your wallet automatically synchronizes with the Bitcoin network after a connection is established, but this process has not completed yet.</source>
+        <translation>Kuvatav info ei pruugi olla ajakohane. Ühenduse loomisel süngitakse sinu rahakott automaatselt Bitconi võrgustikuga, kuid see toiming on hetkel lõpetamata.</translation>
     </message>
     <message>
         <source>Pending:</source>
@@ -856,7 +881,18 @@
     </message>
     <message>
         <source>Recent transactions</source>
-        <translation>Hiljutised tehingud</translation>
+        <translation>Hiljutised transaktsioonid</translation>
+    </message>
+    </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    <message>
+        <source>Dialog</source>
+        <translation>Dialoog</translation>
+    </message>
+    <message>
+        <source>or</source>
+        <translation>või</translation>
     </message>
     </context>
 <context>
@@ -866,16 +902,12 @@
         <translation>Maksepäringu tõrge</translation>
     </message>
     <message>
-        <source>Payment request rejected</source>
-        <translation>Maksepäring tagasi lükatud</translation>
+        <source>Cannot start bitcoin: click-to-pay handler</source>
+        <translation>Bitcoin ei käivitu: vajuta-maksa toiming</translation>
     </message>
     <message>
-        <source>Payment request expired.</source>
-        <translation>Maksepäring aegunud.</translation>
-    </message>
-    <message>
-        <source>Unverified payment requests to custom payment scripts are unsupported.</source>
-        <translation>Kinnitamata maksepäringud kohandatud makse scriptidele ei ole toetatud.</translation>
+        <source>URI handling</source>
+        <translation>URI käsitsemine</translation>
     </message>
     </context>
 <context>
@@ -886,7 +918,7 @@
     </message>
     <message>
         <source>Received</source>
-        <translation>Vastuvõetud</translation>
+        <translation>Vastu võetud</translation>
     </message>
 </context>
 <context>
@@ -944,11 +976,24 @@
         <translation>tundmatu</translation>
     </message>
 </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
+    <message>
+        <source>&amp;Save Image...</source>
+        <translation>&amp;Salvesta Pilt...</translation>
+    </message>
+    <message>
+        <source>&amp;Copy Image</source>
+        <translation>&amp;Kopeeri Pilt</translation>
+    </message>
+    <message>
+        <source>Resulting URI too long, try to reduce the text for label / message.</source>
+        <translation>URI liiga pikk, proovi vähendada märke / sõnumi pikkust.</translation>
+    </message>
+    <message>
+        <source>Error encoding URI into QR Code.</source>
+        <translation>Tõrge URI'st QR koodi loomisel</translation>
+    </message>
     <message>
         <source>Save QR Code</source>
         <translation>Salvesta QR Kood</translation>
@@ -968,10 +1013,6 @@
         <source>&amp;Information</source>
         <translation>&amp;Informatsioon</translation>
     </message>
-    <message>
-        <source>Debug window</source>
-        <translation>Debugimise aken</translation>
-    </message>
     <message>
         <source>General</source>
         <translation>Üldine</translation>
@@ -986,7 +1027,7 @@
     </message>
     <message>
         <source>Network</source>
-        <translation>Võrgustik</translation>
+        <translation>Võrk</translation>
     </message>
     <message>
         <source>Name</source>
@@ -998,11 +1039,7 @@
     </message>
     <message>
         <source>Block chain</source>
-        <translation>Ploki jada</translation>
-    </message>
-    <message>
-        <source>Current number of blocks</source>
-        <translation>Plokkide hetkearv</translation>
+        <translation>Blokiahel</translation>
     </message>
     <message>
         <source>Memory usage</source>
@@ -1014,7 +1051,7 @@
     </message>
     <message>
         <source>Received</source>
-        <translation>Vastuvõetud</translation>
+        <translation>Vastu võetud</translation>
     </message>
     <message>
         <source>Sent</source>
@@ -1040,6 +1077,10 @@
         <source>Services</source>
         <translation>Teenused</translation>
     </message>
+    <message>
+        <source>Ping Time</source>
+        <translation>Pingi Aeg</translation>
+    </message>
     <message>
         <source>Last block time</source>
         <translation>Viimane ploki aeg</translation>
@@ -1052,6 +1093,10 @@
         <source>&amp;Console</source>
         <translation>&amp;Konsool</translation>
     </message>
+    <message>
+        <source>&amp;Network Traffic</source>
+        <translation>&amp;Võrgu Liiklus</translation>
+    </message>
     <message>
         <source>Debug log file</source>
         <translation>Silumise logifail</translation>
@@ -1060,6 +1105,10 @@
         <source>Clear console</source>
         <translation>Puhasta konsool</translation>
     </message>
+    <message>
+        <source>never</source>
+        <translation>mitte kunagi</translation>
+    </message>
     <message>
         <source>Inbound</source>
         <translation>Sisenev</translation>
@@ -1068,14 +1117,6 @@
         <source>Outbound</source>
         <translation>Väljuv</translation>
     </message>
-    <message>
-        <source>Yes</source>
-        <translation>Jah</translation>
-    </message>
-    <message>
-        <source>No</source>
-        <translation>Ei</translation>
-    </message>
     <message>
         <source>Unknown</source>
         <translation>Teadmata</translation>
@@ -1085,7 +1126,7 @@
     <name>ReceiveCoinsDialog</name>
     <message>
         <source>&amp;Amount:</source>
-        <translation>&amp;Summa:</translation>
+        <translation>&amp;Kogus:</translation>
     </message>
     <message>
         <source>&amp;Label:</source>
@@ -1117,51 +1158,43 @@
     </message>
     <message>
         <source>Copy amount</source>
-        <translation>Kopeeri summa</translation>
+        <translation>Kopeeri kogus</translation>
     </message>
-</context>
+    <message>
+        <source>Could not unlock wallet.</source>
+        <translation>Rahakoti lahtilukustamine ebaõnnestus.</translation>
+    </message>
+    </context>
 <context>
     <name>ReceiveRequestDialog</name>
     <message>
-        <source>QR Code</source>
-        <translation>QR Kood</translation>
+        <source>Amount:</source>
+        <translation>Kogus</translation>
+    </message>
+    <message>
+        <source>Label:</source>
+        <translation>Märgis:</translation>
+    </message>
+    <message>
+        <source>Message:</source>
+        <translation>Sõnum:</translation>
+    </message>
+    <message>
+        <source>Wallet:</source>
+        <translation>Rahakott:</translation>
     </message>
     <message>
         <source>Copy &amp;Address</source>
         <translation>&amp;Kopeeri Aadress</translation>
     </message>
+    <message>
+        <source>&amp;Save Image...</source>
+        <translation>&amp;Salvesta Pilt...</translation>
+    </message>
     <message>
         <source>Payment information</source>
         <translation>Makse Informatsioon</translation>
     </message>
-    <message>
-        <source>Address</source>
-        <translation>Aadress</translation>
-    </message>
-    <message>
-        <source>Amount</source>
-        <translation>Summa</translation>
-    </message>
-    <message>
-        <source>Label</source>
-        <translation>Silt</translation>
-    </message>
-    <message>
-        <source>Message</source>
-        <translation>Sõnum</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Rahakott</translation>
-    </message>
-    <message>
-        <source>Resulting URI too long, try to reduce the text for label / message.</source>
-        <translation>URI liiga pikk, proovi vähendada märke / sõnumi pikkust.</translation>
-    </message>
-    <message>
-        <source>Error encoding URI into QR Code.</source>
-        <translation>Tõrge URI'st QR koodi loomisel</translation>
-    </message>
 </context>
 <context>
     <name>RecentRequestsTableModel</name>
@@ -1179,7 +1212,7 @@
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märge puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
     <message>
         <source>(no message)</source>
@@ -1214,7 +1247,7 @@
     </message>
     <message>
         <source>Amount:</source>
-        <translation>Summa:</translation>
+        <translation>Kogus</translation>
     </message>
     <message>
         <source>Fee:</source>
@@ -1286,7 +1319,7 @@
     </message>
     <message>
         <source>Copy amount</source>
-        <translation>Kopeeri summa</translation>
+        <translation>Kopeeri kogus</translation>
     </message>
     <message>
         <source>Copy fee</source>
@@ -1341,12 +1374,12 @@
         <translation>Maksepäring aegunud.</translation>
     </message>
     <message>
-        <source>Warning: Invalid Litecoin address</source>
-        <translation>Hoiatus: Ebakorrektne Litecoin aadress</translation>
+        <source>Warning: Invalid Bitcoin address</source>
+        <translation>Hoiatus: Ebakorrektne Bitcoin aadress</translation>
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märgis puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
 </context>
 <context>
@@ -1391,18 +1424,7 @@
         <source>Pay To:</source>
         <translation>Maksa :</translation>
     </message>
-    <message>
-        <source>Enter a label for this address to add it to your address book</source>
-        <translation>Aadressiraamatusse sisestamiseks märgista aadress</translation>
-    </message>
-</context>
-<context>
-    <name>SendConfirmationDialog</name>
-    <message>
-        <source>Yes</source>
-        <translation>Jah</translation>
-    </message>
-</context>
+    </context>
 <context>
     <name>ShutdownWindow</name>
     <message>
@@ -1425,8 +1447,8 @@
         <translation>&amp;Allkirjastamise teade</translation>
     </message>
     <message>
-        <source>The Litecoin address to sign the message with</source>
-        <translation>Litecoin aadress millega sõnum allkirjastada</translation>
+        <source>The Bitcoin address to sign the message with</source>
+        <translation>Bitcoin aadress millega sõnum allkirjastada</translation>
     </message>
     <message>
         <source>Choose previously used address</source>
@@ -1450,15 +1472,15 @@
     </message>
     <message>
         <source>Signature</source>
-        <translation>Signatuur</translation>
+        <translation>Allkiri</translation>
     </message>
     <message>
         <source>Copy the current signature to the system clipboard</source>
         <translation>Kopeeri praegune signatuur vahemällu</translation>
     </message>
     <message>
-        <source>Sign the message to prove you own this Litecoin address</source>
-        <translation>Allkirjasta sõnum Litecoini aadressi sulle kuulumise tõestamiseks</translation>
+        <source>Sign the message to prove you own this Bitcoin address</source>
+        <translation>Allkirjasta sõnum Bitcoini aadressi sulle kuulumise tõestamiseks</translation>
     </message>
     <message>
         <source>Sign &amp;Message</source>
@@ -1477,12 +1499,12 @@
         <translation>&amp;Kinnita Sõnum</translation>
     </message>
     <message>
-        <source>The Litecoin address the message was signed with</source>
-        <translation>Litecoin aadress millega sõnum on allkirjastatud</translation>
+        <source>The Bitcoin address the message was signed with</source>
+        <translation>Bitcoin aadress millega sõnum on allkirjastatud</translation>
     </message>
     <message>
-        <source>Verify the message to ensure it was signed with the specified Litecoin address</source>
-        <translation>Kinnita sõnum tõestamaks selle allkirjastatust määratud Litecoini aadressiga.</translation>
+        <source>Verify the message to ensure it was signed with the specified Bitcoin address</source>
+        <translation>Kinnita sõnum tõestamaks selle allkirjastatust määratud Bitcoini aadressiga.</translation>
     </message>
     <message>
         <source>Verify &amp;Message</source>
@@ -1522,11 +1544,11 @@
     </message>
     <message>
         <source>Message signed.</source>
-        <translation>Sõnum allkirjastatud</translation>
+        <translation>Sõnum allkirjastatud.</translation>
     </message>
     <message>
         <source>The signature could not be decoded.</source>
-        <translation>Allkirja polnud võimalik dekodeerida.</translation>
+        <translation>Allkirja ei õnnestunud dekodeerida.</translation>
     </message>
     <message>
         <source>Please check the signature and try again.</source>
@@ -1545,13 +1567,6 @@
         <translation>Sõnum verifitseeritud.</translation>
     </message>
 </context>
-<context>
-    <name>SplashScreen</name>
-    <message>
-        <source>[testnet]</source>
-        <translation>[testnet]</translation>
-    </message>
-</context>
 <context>
     <name>TrafficGraphWidget</name>
     <message>
@@ -1639,7 +1654,7 @@
     </message>
     <message>
         <source>Transaction ID</source>
-        <translation>Tehingu ID</translation>
+        <translation>Transaktsiooni ID</translation>
     </message>
     <message>
         <source>Merchant</source>
@@ -1659,7 +1674,7 @@
     </message>
     <message>
         <source>Amount</source>
-        <translation>Summa</translation>
+        <translation>Kogus</translation>
     </message>
     <message>
         <source>true</source>
@@ -1768,7 +1783,7 @@
     </message>
     <message>
         <source>This month</source>
-        <translation>Käesolev kuu</translation>
+        <translation>Käimasolev kuu</translation>
     </message>
     <message>
         <source>Last month</source>
@@ -1776,7 +1791,7 @@
     </message>
     <message>
         <source>This year</source>
-        <translation>Käesolev aasta</translation>
+        <translation>Käimasolev aasta</translation>
     </message>
     <message>
         <source>Range...</source>
@@ -1820,7 +1835,7 @@
     </message>
     <message>
         <source>Copy transaction ID</source>
-        <translation>Kopeeri tehingu ID</translation>
+        <translation>Kopeeri transaktsiooni ID</translation>
     </message>
     <message>
         <source>Edit label</source>
@@ -1864,7 +1879,7 @@
     </message>
     <message>
         <source>Range:</source>
-        <translation>Ulatus:</translation>
+        <translation>Vahemik:</translation>
     </message>
     <message>
         <source>to</source>
@@ -1879,7 +1894,11 @@
     </context>
 <context>
     <name>WalletFrame</name>
-    </context>
+    <message>
+        <source>Create a new wallet</source>
+        <translation>Loo uus rahakott</translation>
+    </message>
+</context>
 <context>
     <name>WalletModel</name>
     <message>
@@ -1897,17 +1916,21 @@
         <source>Export the data in the current tab to a file</source>
         <translation>Ekspordi kuvatava vahelehe sisu faili</translation>
     </message>
+    <message>
+        <source>Error</source>
+        <translation>Viga</translation>
+    </message>
     <message>
         <source>Backup Wallet</source>
-        <translation>Varundatud Rahakott</translation>
+        <translation>Varunda Rahakott</translation>
     </message>
     <message>
         <source>Wallet Data (*.dat)</source>
-        <translation>Rahakoti andmed (*.dat)</translation>
+        <translation>Rahakoti Andmed (*.dat)</translation>
     </message>
     <message>
         <source>Backup Failed</source>
-        <translation>Varundamine nurjus</translation>
+        <translation>Varundamine Ebaõnnestus</translation>
     </message>
     <message>
         <source>Backup Successful</source>
@@ -1916,10 +1939,6 @@
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Litecoin Core</source>
-        <translation>Litecoini tuumik</translation>
-    </message>
     <message>
         <source>This is a pre-release test build - use at your own risk - do not use for mining or merchant applications</source>
         <translation>See on test-versioon - kasutamine omal riisikol - ära kasuta mining'uks ega kaupmeeste programmides</translation>
@@ -1948,10 +1967,6 @@
         <source>Error opening block database</source>
         <translation>Tõrge bloki andmebaasi avamisel</translation>
     </message>
-    <message>
-        <source>Error: Disk space is low!</source>
-        <translation>Tõrge: liiga vähe kettaruumi!</translation>
-    </message>
     <message>
         <source>Failed to listen on any port. Use -listen=0 if you want this.</source>
         <translation>Pordi kuulamine nurjus. Soovikorral kasuta -listen=0.</translation>
@@ -1960,10 +1975,6 @@
         <source>Verifying blocks...</source>
         <translation>Kontrollin blokke...</translation>
     </message>
-    <message>
-        <source>Information</source>
-        <translation>Informatsioon</translation>
-    </message>
     <message>
         <source>Signing transaction failed</source>
         <translation>Tehingu allkirjastamine ebaõnnestus</translation>
@@ -1980,10 +1991,6 @@
         <source>Transaction too large</source>
         <translation>Tehing liiga suur</translation>
     </message>
-    <message>
-        <source>Warning</source>
-        <translation>Hoiatus</translation>
-    </message>
     <message>
         <source>Unknown network specified in -onlynet: '%s'</source>
         <translation>Kirjeldatud tundmatu võrgustik -onlynet'is: '%s'</translation>
@@ -2012,9 +2019,5 @@
         <source>Done loading</source>
         <translation>Laetud</translation>
     </message>
-    <message>
-        <source>Error</source>
-        <translation>Tõrge</translation>
-    </message>
 </context>
 </TS>
\ No newline at end of file
