@@ -89,10 +89,6 @@
         <source>Open &amp;URI...</source>
         <translation>Vhura &amp;URI</translation>
     </message>
-    <message>
-        <source>Litecoin</source>
-        <translation>Litecoin</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Tumira</translation>
@@ -130,7 +126,7 @@
     <name>CoinControlDialog</name>
     <message>
         <source>Amount</source>
-        <translation>Marii </translation>
+        <translation>Marii</translation>
     </message>
     <message>
         <source>Date</source>
@@ -141,6 +137,12 @@
         <translation>(hapana zita)</translation>
     </message>
     </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     </context>
@@ -153,8 +155,8 @@
 <context>
     <name>Intro</name>
     <message>
-        <source>Litecoin</source>
-        <translation>Litecoin</translation>
+        <source>Bitcoin</source>
+        <translation>Bitcoin</translation>
     </message>
     </context>
 <context>
@@ -163,12 +165,18 @@
 <context>
     <name>OpenURIDialog</name>
     </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     </context>
 <context>
     <name>OverviewPage</name>
     </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    </context>
 <context>
     <name>PaymentServer</name>
     </context>
@@ -179,11 +187,11 @@
     <name>QObject</name>
     <message>
         <source>Amount</source>
-        <translation>Marii </translation>
+        <translation>Marii</translation>
     </message>
     <message>
-        <source>Enter a Litecoin address (e.g. %1)</source>
-        <translation>Nyora kero ye Litecoin (sekuti %1)</translation>
+        <source>Enter a Bitcoin address (e.g. %1)</source>
+        <translation>Nyora kero ye Bitcoin (sekuti %1)</translation>
     </message>
     <message>
         <source>%1 d</source>
@@ -207,12 +215,9 @@
     </message>
     <message>
         <source>N/A</source>
-        <translation>Hapana </translation>
+        <translation>Hapana</translation>
     </message>
     </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
     </context>
@@ -220,7 +225,7 @@
     <name>RPCConsole</name>
     <message>
         <source>N/A</source>
-        <translation>Hapana </translation>
+        <translation>Hapana</translation>
     </message>
     </context>
 <context>
@@ -228,22 +233,6 @@
     </context>
 <context>
     <name>ReceiveRequestDialog</name>
-    <message>
-        <source>Address</source>
-        <translation>Kero</translation>
-    </message>
-    <message>
-        <source>Amount</source>
-        <translation>Marii </translation>
-    </message>
-    <message>
-        <source>Label</source>
-        <translation>Zita</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Chikwama</translation>
-    </message>
     </context>
 <context>
     <name>RecentRequestsTableModel</name>
@@ -270,18 +259,12 @@
 <context>
     <name>SendCoinsEntry</name>
     </context>
-<context>
-    <name>SendConfirmationDialog</name>
-    </context>
 <context>
     <name>ShutdownWindow</name>
     </context>
 <context>
     <name>SignVerifyMessageDialog</name>
     </context>
-<context>
-    <name>SplashScreen</name>
-    </context>
 <context>
     <name>TrafficGraphWidget</name>
     </context>
@@ -293,7 +276,7 @@
     </message>
     <message>
         <source>Amount</source>
-        <translation>Marii </translation>
+        <translation>Marii</translation>
     </message>
     </context>
 <context>
@@ -346,13 +329,5 @@
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Information</source>
-        <translation>Ruzivo</translation>
-    </message>
-    <message>
-        <source>Warning</source>
-        <translation>Hokoyo</translation>
-    </message>
     </context>
 </TS>
\ No newline at end of file
