@@ -19,13 +19,15 @@ public:
     void clear();
     void setCheckValidator(const QValidator *v);
     bool isValid();
+    void setValidating(bool validating);
 
 protected:
-    void focusInEvent(QFocusEvent *evt);
-    void focusOutEvent(QFocusEvent *evt);
+    void focusInEvent(QFocusEvent *evt) override;
+    void focusOutEvent(QFocusEvent *evt) override;
 
 private:
     bool valid;
+    bool validating;
     const QValidator *checkValidator;
 
 public Q_SLOTS:
