@@ -1,11 +1,11 @@
-// Copyright (c) 2012-2018 The Bitcoin Core developers
+// Copyright (c) 2012-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #include <dbwrapper.h>
+#include <test/util/setup_common.h>
 #include <uint256.h>
-#include <random.h>
-#include <test/test_bitcoin.h>
+#include <util/memory.h>
 
 #include <memory>
 
@@ -27,7 +27,7 @@ BOOST_AUTO_TEST_CASE(dbwrapper)
 {
     // Perform tests both obfuscated and non-obfuscated.
     for (const bool obfuscate : {false, true}) {
-        fs::path ph = SetDataDir(std::string("dbwrapper").append(obfuscate ? "_true" : "_false"));
+        fs::path ph = GetDataDir() / (obfuscate ? "dbwrapper_obfuscate_true" : "dbwrapper_obfuscate_false");
         CDBWrapper dbw(ph, (1 << 20), true, false, obfuscate);
         char key = 'k';
         uint256 in = InsecureRand256();
@@ -42,12 +42,92 @@ BOOST_AUTO_TEST_CASE(dbwrapper)
     }
 }
 
+BOOST_AUTO_TEST_CASE(dbwrapper_basic_data)
+{
+    // Perform tests both obfuscated and non-obfuscated.
+    for (bool obfuscate : {false, true}) {
+        fs::path ph = GetDataDir() / (obfuscate ? "dbwrapper_1_obfuscate_true" : "dbwrapper_1_obfuscate_false");
+        CDBWrapper dbw(ph, (1 << 20), false, true, obfuscate);
+
+        uint256 res;
+        uint32_t res_uint_32;
+        bool res_bool;
+
+        // Ensure that we're doing real obfuscation when obfuscate=true
+        BOOST_CHECK(obfuscate != is_null_key(dbwrapper_private::GetObfuscateKey(dbw)));
+
+        //Simulate block raw data - "b + block hash"
+        std::string key_block = "b" + InsecureRand256().ToString();
+
+        uint256 in_block = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_block, in_block));
+        BOOST_CHECK(dbw.Read(key_block, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_block.ToString());
+
+        //Simulate file raw data - "f + file_number"
+        std::string key_file = strprintf("f%04x", InsecureRand32());
+
+        uint256 in_file_info = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_file, in_file_info));
+        BOOST_CHECK(dbw.Read(key_file, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_file_info.ToString());
+
+        //Simulate transaction raw data - "t + transaction hash"
+        std::string key_transaction = "t" + InsecureRand256().ToString();
+
+        uint256 in_transaction = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_transaction, in_transaction));
+        BOOST_CHECK(dbw.Read(key_transaction, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_transaction.ToString());
+
+        //Simulate UTXO raw data - "c + transaction hash"
+        std::string key_utxo = "c" + InsecureRand256().ToString();
+
+        uint256 in_utxo = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_utxo, in_utxo));
+        BOOST_CHECK(dbw.Read(key_utxo, res));
+        BOOST_CHECK_EQUAL(res.ToString(), in_utxo.ToString());
+
+        //Simulate last block file number - "l"
+        char key_last_blockfile_number = 'l';
+        uint32_t lastblockfilenumber = InsecureRand32();
+        BOOST_CHECK(dbw.Write(key_last_blockfile_number, lastblockfilenumber));
+        BOOST_CHECK(dbw.Read(key_last_blockfile_number, res_uint_32));
+        BOOST_CHECK_EQUAL(lastblockfilenumber, res_uint_32);
+
+        //Simulate Is Reindexing - "R"
+        char key_IsReindexing = 'R';
+        bool isInReindexing = InsecureRandBool();
+        BOOST_CHECK(dbw.Write(key_IsReindexing, isInReindexing));
+        BOOST_CHECK(dbw.Read(key_IsReindexing, res_bool));
+        BOOST_CHECK_EQUAL(isInReindexing, res_bool);
+
+        //Simulate last block hash up to which UXTO covers - 'B'
+        char key_lastblockhash_uxto = 'B';
+        uint256 lastblock_hash = InsecureRand256();
+        BOOST_CHECK(dbw.Write(key_lastblockhash_uxto, lastblock_hash));
+        BOOST_CHECK(dbw.Read(key_lastblockhash_uxto, res));
+        BOOST_CHECK_EQUAL(lastblock_hash, res);
+
+        //Simulate file raw data - "F + filename_number + filename"
+        std::string file_option_tag = "F";
+        uint8_t filename_length = InsecureRandBits(8);
+        std::string filename = "randomfilename";
+        std::string key_file_option = strprintf("%s%01x%s", file_option_tag,filename_length,filename);
+
+        bool in_file_bool = InsecureRandBool();
+        BOOST_CHECK(dbw.Write(key_file_option, in_file_bool));
+        BOOST_CHECK(dbw.Read(key_file_option, res_bool));
+        BOOST_CHECK_EQUAL(res_bool, in_file_bool);
+   }
+}
+
 // Test batch operations
 BOOST_AUTO_TEST_CASE(dbwrapper_batch)
 {
     // Perform tests both obfuscated and non-obfuscated.
     for (const bool obfuscate : {false, true}) {
-        fs::path ph = SetDataDir(std::string("dbwrapper_batch").append(obfuscate ? "_true" : "_false"));
+        fs::path ph = GetDataDir() / (obfuscate ? "dbwrapper_batch_obfuscate_true" : "dbwrapper_batch_obfuscate_false");
         CDBWrapper dbw(ph, (1 << 20), true, false, obfuscate);
 
         char key = 'i';
@@ -83,7 +163,7 @@ BOOST_AUTO_TEST_CASE(dbwrapper_iterator)
 {
     // Perform tests both obfuscated and non-obfuscated.
     for (const bool obfuscate : {false, true}) {
-        fs::path ph = SetDataDir(std::string("dbwrapper_iterator").append(obfuscate ? "_true" : "_false"));
+        fs::path ph = GetDataDir() / (obfuscate ? "dbwrapper_iterator_obfuscate_true" : "dbwrapper_iterator_obfuscate_false");
         CDBWrapper dbw(ph, (1 << 20), true, false, obfuscate);
 
         // The two keys are intentionally chosen for ordering
@@ -123,7 +203,7 @@ BOOST_AUTO_TEST_CASE(dbwrapper_iterator)
 BOOST_AUTO_TEST_CASE(existing_data_no_obfuscate)
 {
     // We're going to share this fs::path between two wrappers
-    fs::path ph = SetDataDir("existing_data_no_obfuscate");
+    fs::path ph = GetDataDir() / "existing_data_no_obfuscate";
     create_directories(ph);
 
     // Set up a non-obfuscated wrapper to write some initial data.
@@ -164,7 +244,7 @@ BOOST_AUTO_TEST_CASE(existing_data_no_obfuscate)
 BOOST_AUTO_TEST_CASE(existing_data_reindex)
 {
     // We're going to share this fs::path between two wrappers
-    fs::path ph = SetDataDir("existing_data_reindex");
+    fs::path ph = GetDataDir() / "existing_data_reindex";
     create_directories(ph);
 
     // Set up a non-obfuscated wrapper to write some initial data.
@@ -199,7 +279,7 @@ BOOST_AUTO_TEST_CASE(existing_data_reindex)
 
 BOOST_AUTO_TEST_CASE(iterator_ordering)
 {
-    fs::path ph = SetDataDir("iterator_ordering");
+    fs::path ph = GetDataDir() / "iterator_ordering";
     CDBWrapper dbw(ph, (1 << 20), true, false, false);
     for (int x=0x00; x<256; ++x) {
         uint8_t key = x;
@@ -251,24 +331,26 @@ struct StringContentsSerializer {
     }
     StringContentsSerializer& operator+=(const StringContentsSerializer& s) { return *this += s.str; }
 
-    ADD_SERIALIZE_METHODS;
+    template<typename Stream>
+    void Serialize(Stream& s) const
+    {
+        for (size_t i = 0; i < str.size(); i++) {
+            s << str[i];
+        }
+    }
 
-    template <typename Stream, typename Operation>
-    inline void SerializationOp(Stream& s, Operation ser_action) {
-        if (ser_action.ForRead()) {
-            str.clear();
-            char c = 0;
-            while (true) {
-                try {
-                    READWRITE(c);
-                    str.push_back(c);
-                } catch (const std::ios_base::failure&) {
-                    break;
-                }
+    template<typename Stream>
+    void Unserialize(Stream& s)
+    {
+        str.clear();
+        char c = 0;
+        while (true) {
+            try {
+                s >> c;
+                str.push_back(c);
+            } catch (const std::ios_base::failure&) {
+                break;
             }
-        } else {
-            for (size_t i = 0; i < str.size(); i++)
-                READWRITE(str[i]);
         }
     }
 };
@@ -277,7 +359,7 @@ BOOST_AUTO_TEST_CASE(iterator_string_ordering)
 {
     char buf[10];
 
-    fs::path ph = SetDataDir("iterator_string_ordering");
+    fs::path ph = GetDataDir() / "iterator_string_ordering";
     CDBWrapper dbw(ph, (1 << 20), true, false, false);
     for (int x=0x00; x<10; ++x) {
         for (int y = 0; y < 10; y++) {
@@ -317,6 +399,18 @@ BOOST_AUTO_TEST_CASE(iterator_string_ordering)
     }
 }
 
+BOOST_AUTO_TEST_CASE(unicodepath)
+{
+    // Attempt to create a database with a UTF8 character in the path.
+    // On Windows this test will fail if the directory is created using
+    // the ANSI CreateDirectoryA call and the code page isn't UTF8.
+    // It will succeed if created with CreateDirectoryW.
+    fs::path ph = GetDataDir() / "test_runner_₿_🏃_20191128_104644";
+    CDBWrapper dbw(ph, (1 << 20));
+
+    fs::path lockPath = ph / "LOCK";
+    BOOST_CHECK(fs::exists(lockPath));
+}
 
 
 BOOST_AUTO_TEST_SUITE_END()
