@@ -1,4 +1,4 @@
-// Copyright (c) 2011-2018 The Bitcoin Core developers
+// Copyright (c) 2011-2020 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -8,8 +8,8 @@
 #include <key.h>
 #include <key_io.h>
 #include <script/script.h>
+#include <test/util/setup_common.h>
 #include <util/strencodings.h>
-#include <test/test_bitcoin.h>
 
 #include <boost/test/unit_test.hpp>
 
