@@ -8,7 +8,7 @@ Tests that a node configured with -prune=2200 signals NODE_NETWORK_LIMITED corre
 and that it responds to getdata requests for blocks correctly:
     - send a block within 288 + 2 of the tip
     - disconnect peers who request blocks older than that."""
-from test_framework.messages import CInv, MSG_BLOCK, msg_getdata, msg_verack, NODE_NETWORK_LIMITED, NODE_WITNESS, NODE_MWEB
+from test_framework.messages import CInv, MSG_BLOCK, msg_getdata, msg_verack, NODE_MWEB, NODE_MWEB_LIGHT_CLIENT, NODE_NETWORK_LIMITED, NODE_WITNESS
 from test_framework.p2p import P2PInterface
 from test_framework.test_framework import BitcoinTestFramework
 from test_framework.util import (
@@ -49,7 +49,7 @@ class NodeNetworkLimitedTest(BitcoinTestFramework):
     def run_test(self):
         node = self.nodes[0].add_p2p_connection(P2PIgnoreInv())
 
-        expected_services = NODE_WITNESS | NODE_MWEB | NODE_NETWORK_LIMITED
+        expected_services = NODE_WITNESS | NODE_MWEB | NODE_MWEB_LIGHT_CLIENT | NODE_NETWORK_LIMITED
 
         self.log.info("Check that node has signalled expected services.")
         assert_equal(node.nServices, expected_services)
